@@ -6,11 +6,14 @@ import string
 PUNCTUATION_REGEX = re.compile('[{0}]'.format(re.escape(string.punctuation)))
 
 
-def strip_punc(s):
+def strip_punc(s, all=False):
     '''Removes punctuation from a string.'''
-    return PUNCTUATION_REGEX.sub('', s)
+    if all:
+        return PUNCTUATION_REGEX.sub('', s.strip())
+    else:
+        return s.strip().strip(string.punctuation)
 
 
-def lowerstrip(s):
+def lowerstrip(s, all=False):
     '''Makes text all lowercase and strips punctuation and whitespace.'''
-    return strip_punc(s.lower().strip())
+    return strip_punc(s.lower().strip(), all=all)
